package cpp_flowchart;

import java.awt.Color;
import java.awt.Insets;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

@SuppressWarnings("serial")
public class CppFlowchartPanel extends JPanel {
	CppFlowchart cppflowchart;
	VariablePanel variablePanel;
	CodePanel codePanel;
	SourcePanel sourcepanel;
	FlowchartPanel flowchartPanel;
	String code;


	public CppFlowchartPanel(CppFlowchart pf)
	{
		cppflowchart = pf;
		sourcepanel = new SourcePanel();
		variablePanel = new VariablePanel(this);
		codePanel = new CodePanel(this);
		flowchartPanel = new FlowchartPanel(this);
//		variablePanel.setFlowchart(flowchartPanel.getFlowchart());
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		JPanel subpanel = new JPanel();
		subpanel.setLayout(new BoxLayout(subpanel, BoxLayout.X_AXIS));
		subpanel.add(addBorder(sourcepanel,"パーツ"));
		subpanel.add(variablePanel);
		subpanel.setMaximumSize(subpanel.getPreferredSize());
		panel.add(subpanel);
		panel.add(addBorder(flowchartPanel, "フローチャート"));
		add(panel);
		add(codePanel);
		code = null;
		makeCode();
		refreshCode();
	}
	@Override
	public Insets getInsets() { return new Insets(1,1,1,1);}

	private JComponent addBorder(JComponent c, String title)
	{
		Border lineBorder = BorderFactory.createLineBorder(Color.BLACK);
		Border out = BorderFactory.createTitledBorder(lineBorder, title);
		Border in = BorderFactory.createEmptyBorder(1, 1, 1, 1);
		c.setBorder(BorderFactory.createCompoundBorder(out, in));
		return c;
	}


	public Flowchart makeEmptyFlowchart()
	{
		return new Flowchart(flowchartPanel);
	}

	public void setFlowchart(Flowchart f, String[] vars)
	{
		flowchartPanel.setFlowchart(f);
		variablePanel.setVars(vars);
		makeCode();
	}

	public boolean isDirty()
	{
		return flowchartPanel.isDirty() || variablePanel.isDirty();
	}

	public void makeClean()
	{
		flowchartPanel.makeClean();
		variablePanel.makeClean();
		cppflowchart.setTitle();
		refreshCode();
	}

	public boolean load(String filename)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringComments(true);
		factory.setIgnoringElementContentWhitespace(true);
		try {
			File f = new File(filename);
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(f);
//			penflowchart.setTitle();
			return readXML(doc);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return false;
	}

	public boolean save(String filename)
	{
		TransformerFactory tfactory = TransformerFactory.newInstance();
		try {
			Document doc = makeXML();
			if(doc == null)return false;
			Transformer tformer = tfactory.newTransformer();
			FileOutputStream s = new FileOutputStream(filename);
			tformer.transform(new DOMSource(doc), new StreamResult(s));
			s.close();
			makeClean();
			return true;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (TransformerConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return false;
	}

	public void saveascpp(String filename_c) {
		try {
			FileOutputStream s = new FileOutputStream(filename_c);
			s.write(code.getBytes());
			s.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			// TODO 自動生成された catch ブロック
			e.printStackTrace();
		}
	}

	
	private boolean readXML(Document doc)
	{
		variablePanel.reset();
		flowchartPanel.reset();

		Node root_node = doc.getFirstChild();
		NamedNodeMap map = root_node.getAttributes();
		Node node = map.getNamedItem("int");
		if(node != null) variablePanel.setIntVars(node.getNodeValue());
		node = map.getNamedItem("float");
		if(node != null) variablePanel.setFloatVars(node.getNodeValue());
		node = map.getNamedItem("string");
		if(node != null) variablePanel.setStringVars(node.getNodeValue());
		node = map.getNamedItem("boolean");
		if(node != null) variablePanel.setBooleanVars(node.getNodeValue());
		flowchartPanel.appendParts(root_node);
		repaint();
		makeCode();
		refreshCode();
		makeClean();
		return true;
	}

	private Document makeXML()
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringComments(true);
		factory.setIgnoringElementContentWhitespace(true);
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			Document doc = builder.newDocument();
			doc.setXmlStandalone(true);
			Element root = doc.createElement("root");
			String s;
			s = variablePanel.getIntVars();
			if(!s.equals(""))root.setAttribute("int", s);
			s = variablePanel.getFloatVars();
			if(!s.equals(""))root.setAttribute("float", s);
			s = variablePanel.getStringVars();
			if(!s.equals(""))root.setAttribute("string", s);
			s = variablePanel.getBooleanVars();
			if(!s.equals(""))root.setAttribute("boolean", s);

			flowchartPanel.makeXML(doc, root);
			doc.appendChild(root);
			return doc;
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return null;
	}

	public void makeCode()
	{
		StringBuilder sb = new StringBuilder();
		String var;
		sb.append("#include <iostream>\n");
		sb.append("using namespace std;\n\n");
		sb.append("int main(void){\n");
		var = variablePanel.getIntVars();
		if(var.length() > 0) sb.append("\tint "+ var + ";\n");
		var = variablePanel.getFloatVars();
		if(var.length() > 0) sb.append("\tfloat "+ var + ";\n");
		var = variablePanel.getStringVars();
		if(var.length() > 0) sb.append("\tstring "+ var + ";\n");
		var = variablePanel.getBooleanVars();
		if(var.length() > 0) sb.append("\tboolean "+ var + ";\n");
		flowchartPanel.appendTo(sb);
		sb.append("\treturn 0;\n");
		sb.append("}\n");
		code = sb.toString();
	}

	public void refreshCode()
	{
		codePanel.setCode(code);
//		cppflowchart.refreshPen(code);
	}

	public void reset() {
		flowchartPanel.reset();
		variablePanel.reset();
		makeCode();
		refreshCode();
		cppflowchart.setTitle();
	}

	public void setTitle()
	{
		cppflowchart.setTitle();
	}

	public String getEps(FileOutputStream s, VariablePanel variablePanel) {
		return flowchartPanel.getEps(s, variablePanel);
	}
	public void savePng(String filename_e) {
		flowchartPanel.savePng(filename_e, variablePanel);
	}
}
