package cpp_flowchart;

import java.awt.Dimension;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.FileOutputStream;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.event.MouseInputListener;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;


public class FlowchartPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	private static int action = DnDConstants.ACTION_COPY_OR_MOVE;
	private Flowchart chart;
	private CppFlowchartPanel parent;
	private JButton buttonBig, buttonSmall, buttonReset;
	private ActionListener buttonListener;
//	private TargetPanel parent;

	public FlowchartPanel(final CppFlowchartPanel parent) {
		super();
		this.parent = parent;
		new DropTarget(this, action, listener);
		setPreferredSize(new Dimension(600, 500));
		setLayout(null);
		Parts.resetSize();
		chart = new Flowchart(this);
		addMouseMotionListener(mouse_listener);
		addMouseListener(mouse_listener);
		setLayout(null);
		buttonListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Object source = e.getSource();
				if(source == buttonBig)
				{
					int size = Parts.getSize();
					if(size < 30)
					{
						Parts.setSize(Parts.getSize() + 1);
						parent.repaint();
					}
				}
				else if(source == buttonSmall)
				{
					int size = Parts.getSize();
					if(size > 1)
					{
						Parts.setSize(size - 1);
						parent.repaint();
					}
				}
				else if(source == buttonReset)
				{
					Parts.resetSize();
					parent.repaint();
				}
			}
		};
		buttonBig = new JButton("拡大");
		buttonBig.setBounds(10, 20, 60, 20);
		buttonBig.addActionListener(buttonListener);
		this.add(buttonBig);
		buttonSmall = new JButton("縮小");
		buttonSmall.setBounds(10, 40, 60, 20);
		buttonSmall.addActionListener(buttonListener);
		this.add(buttonSmall);
		buttonReset = new JButton("戻す");
		buttonReset.setBounds(10, 60, 60, 20);
		buttonReset.addActionListener(buttonListener);
		this.add(buttonReset);
	}

	public void setFlowchart(Flowchart chart)
	{
		this.chart = chart;
	}

	public void reset()
	{
		chart.reset();
		chart.repaint();
	}

	public boolean isDirty()
	{
		return chart.isDirty();
	}

	public void makeClean()
	{
		chart.makeClean();
	}

	public void appendParts(Node root_node)
	{
		chart.appendParts(root_node);
	}

	public void appendTo(StringBuilder sb)
	{
		chart.appendCode(sb);
	}

	private MouseInputListener mouse_listener = new MouseInputListener()
	{
		int x0, y0;

		@Override
		public void mouseClicked(MouseEvent e) {
			if(javax.swing.SwingUtilities.isRightMouseButton(e))
				chart.rightButtonClicked(e);
			else if(e.getClickCount() > 1)
				chart.doubleClicked(e);

		}

		@Override
		public void mouseEntered(MouseEvent arg0) {}

		@Override
		public void mouseExited(MouseEvent arg0) {}

		@Override
		public void mousePressed(MouseEvent e) {
			e.consume();
			if(!javax.swing.SwingUtilities.isLeftMouseButton(e))
				return;
			x0 = e.getX();
			y0 = e.getY();
		}

		@Override
		public void mouseReleased(MouseEvent arg0) {}

		@Override
		public void mouseDragged(MouseEvent e) {
			e.consume();
			if(!javax.swing.SwingUtilities.isLeftMouseButton(e))
				return;
			int x = e.getX();
			int y = e.getY();
			int dx = x - x0;
			int dy = y - y0;
			x0 = x;
			y0 = y;
			chart.moveOrigin(dx, dy);
			FlowchartPanel tp = (FlowchartPanel)e.getSource();
			tp.repaint();
		}

		@Override
		public void mouseMoved(MouseEvent e) {
			chart.mouseMoved(e);
		}
	};

	private DropTargetListener listener = new DropTargetListener() {

		@Override
		public void dropActionChanged(DropTargetDragEvent arg0) {}

		@Override
		public void drop(DropTargetDropEvent e) {
			e.acceptDrop(action);
			Transferable tr = e.getTransferable();
			Point p = e.getLocation();
			try
			{
				if(e.isDataFlavorSupported(DataFlavor.stringFlavor))
				{
					String str = (String)tr.getTransferData(DataFlavor.stringFlavor);
					if(str != null)
					{
						if(chart.addParts(str, p))
						{
							repaint();
						}
					}
				}
			}
			catch(Exception ex)
			{
				// do nothing
			}
			finally
			{
				e.dropComplete(true);
			}
		}

		@Override
		public void dragOver(DropTargetDragEvent arg0) {
			// barをlistにいれておいて，近くかどうかをアレするというのはどうか
		}

		@Override
		public void dragExit(DropTargetEvent arg0) {}

		@Override
		public void dragEnter(DropTargetDragEvent arg0) {}
	};

	@Override
	public void paint(Graphics g)
	{
		super.paint(g);
		chart.paint(g);
	}

	public String getEps(FileOutputStream s, VariablePanel variablePanel)
	{
		return chart.getEps(s, variablePanel);
	}

	public void makeXML(Document doc, Element root) {
		chart.makeXML(doc, root);
	}

	public Flowchart getFlowchart() {
		return chart;
	}

	public void refreshCode()
	{
		parent.makeCode();
		parent.refreshCode();
	}

	public void setTitle() {
		parent.setTitle();

	}

	public void savePng(String filename_e, VariablePanel variablePanel) {
		chart.savePng(filename_e, variablePanel);
	}

/*
	public void writePdf(FileOutputStream s) {
		chart.writePdf(s);
	}
*/
}
