package cpp_flowchart;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
//import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;

//import com.itextpdf.text.pdf.PdfContentByte;

abstract public class Parts_LoopBegin extends Parts {
	protected int text_width, text_height;
	protected Parts_LoopEnd loopend;
	protected boolean hilight;
//	private Rectangle frame;

	public Parts_LoopBegin()
	{
		super();
		hilight = false;
	}

	public void setHilight(boolean b)
	{
		hilight = b;
	}

	public void setLoopEnd(Parts_LoopEnd end)
	{
		loopend = end;
	}

	@Override
	public Parts getBlockEnd()
	{
		return loopend;
	}

	public Parts_LoopEnd getLoopEnd()
	{
		return loopend;
	}

	abstract void setText2(String s);
	abstract String getText2();

	@Override
	public boolean isRecursiveBegin()
	{
		return true;
	}

	public boolean hasText()
	{
		return false;
	}

	@Override
	public int getHeight()
	{
		return text_height * (hasText() ? 2 : 1) + size * 2;
	}

	@Override
	public int getWidth()
	{
		return text_width + size * 2;
	}

	@Override
	public Parts paint(Graphics g, FontMetrics fm, Point p) {
//		g.setColor(Color.LIGHT_GRAY);
//		g.drawRect(frame.x + p.x - 2, frame.y + p.y - 2, frame.width + 4, frame.height + 4);
		setX1(p.x - getWidth() / 2);
		setX2(p.x + getWidth() / 2);
		setY1(p.y);
		setY2(p.y + getHeight());
		int x[] = {getX1(), getX1() + size * 2, getX2() - size * 2, getX2(), getX2(), getX1()};
		int y[] = {getY1() + size * 2, getY1(), getY1(), getY1() + size * 2, getY2(), getY2()};
		g.setColor(hilight ? Color.YELLOW : Color.WHITE);
		g.fillPolygon(x,y,6);
		g.setColor(Color.BLACK);
		if(hasText())
		{
			int ym = (getY1() + getY2()) / 2;
			drawString(getText(),g,fm,getX1(),getY1(),getX2(),ym);
			drawString(getText2(),g,fm,getX1(),ym,getX2(),getY2());
		}
		g.drawPolygon(x, y, 6);

		p.y = getY2();

		Parts pNext = getLeft().paint(g, fm, p);
		return pNext.getLeft().paint(g, fm, p);
	}
/*
	@Override
	public Parts paintPdf(PdfContentByte cb, int height, Graphics g,
			FontMetrics fm, Point p) {
		setX1(p.x - getWidth() / 2);
		setX2(p.x + getWidth() / 2);
		setY1(p.y);
		setY2(p.y + getHeight());
		int x[] = {getX1(), getX1() + 10, getX2() - 10, getX2(), getX2(), getX1()};
		int y[] = {getY1() + 10, getY1(), getY1(), getY1() + 10, getY2(), getY2()};
		g.setColor(Color.WHITE);
		g.fillPolygon(x,y,6);
		g.setColor(Color.BLACK);
		if(hasText())
		{
			int ym = (getY1() + getY2()) / 2;
			drawStringPdf(cb, height, getText(),g,fm,getX1(),getY1(),getX2(),ym);
			drawStringPdf(cb, height, getText2(),g,fm,getX1(),ym,getX2(),getY2());
		}
		g.drawPolygon(x, y, 6);

		p.y = getY2();

		Parts pNext = getLeft().paintPdf(cb, height, g, fm, p);
		return pNext.getLeft().paintPdf(cb, height, g, fm, p);
	}
*/
	@Override
	public void setSize(Graphics g, FontMetrics fm) {
		text_width = text_height = 0;
		if(hasText() && getText() != null)
		{
			Rectangle2D rect = fm.getStringBounds(getText(), g);
			text_width = (int)rect.getWidth();
			text_height = (int)rect.getHeight();
			if(getText2() != null)
			{
				rect = fm.getStringBounds(getText2(), g);
				int tw = (int)rect.getWidth();
				int th = (int)rect.getHeight();
				if(tw > text_width) text_width = tw;
				if(th > text_height) text_height = th;
			}
		}
		if(loopend.hasText() && loopend.getText() != null)
		{
			Rectangle2D rect = fm.getStringBounds(loopend.getText(), g);
			text_width = (int)rect.getWidth();
			text_height = (int)rect.getHeight();
			if(loopend.getText2() != null)
			{
				rect = fm.getStringBounds(loopend.getText2(), g);
				int tw = (int)rect.getWidth();
				int th = (int)rect.getHeight();
				if(tw > text_width) text_width = tw;
				if(th > text_height) text_height = th;
			}
		}
		if(text_width < size * 6) text_width = size * 6;
		if(text_height < size * 2) text_height = size * 2;
		loopend.setTextSize(text_height, text_width);
	}

	@Override
	public Parts calcSize(Graphics g, FontMetrics fm, Point p, Point p1, Point p2)
	{
		setSize(g, fm);
//		int prev_y = p.y;
		int x1 = p.x - getWidth() / 2 ;
		int x2 = p.x + getWidth() / 2 ;
		if(x1 < p1.x) p1.x = x1;
		if(x2 > p2.x) p2.x = x2;
		p2.y += getHeight();
		p.y += getHeight();
		p1.x -= 2; p2.x += 2;
		Parts rtnv = getLeft().calcSize(g, fm, p, p1, p2);
//		frame = new Rectangle(p1.x - p.x, 0, p2.x - p1.x, loopend.getY2() - prev_y);
		return rtnv;
	}

	@Override
	public boolean deleteMe(Flowchart chart)
	{
		Parts parent = chart.findParent(this);
		if(parent != null)
		{
			parent.setLeft(loopend.left.left);
			return true;
		}
		return false;
	}

}
